@@ -16,7 +16,7 @@ See http://docs.openstack.org/developer/oslo.i18n/usage.html
 
 """
 
-import oslo.i18n
+import oslo_i18n
 
 
 # NOTE(dhellmann): This reference to o-s-l-o will be replaced by the
@@ -24,7 +24,7 @@ import oslo.i18n
 # repository. It is OK to have more than one translation function
 # using the same domain, since there will still only be one message
 # catalog.
-_translators = oslo.i18n.TranslatorFactory(domain='keystoneclient')
+_translators = oslo_i18n.TranslatorFactory(domain='keystoneclient')
 
 # The primary translation function using the well-known name "_"
 _ = _translators.primary
