@@ -12,6 +12,7 @@
 
 import argparse
 import itertools
+import logging
 import uuid
 
 import mock
@@ -25,6 +26,7 @@ from testtools import matchers
 from keystoneclient import adapter
 from keystoneclient.auth import base
 from keystoneclient import exceptions
+from keystoneclient.i18n import _
 from keystoneclient import session as client_session
 from keystoneclient.tests.unit import utils
 
@@ -38,7 +40,7 @@ class SessionTests(utils.TestCase):
         self.stub_url('GET', text='response')
         resp = session.get(self.TEST_URL)
 
-        self.assertEqual('GET', self.requests.last_request.method)
+        self.assertEqual('GET', self.requests_mock.last_request.method)
         self.assertEqual(resp.text, 'response')
         self.assertTrue(resp.ok)
 
@@ -47,7 +49,7 @@ class SessionTests(utils.TestCase):
         self.stub_url('POST', text='response')
         resp = session.post(self.TEST_URL, json={'hello': 'world'})
 
-        self.assertEqual('POST', self.requests.last_request.method)
+        self.assertEqual('POST', self.requests_mock.last_request.method)
         self.assertEqual(resp.text, 'response')
         self.assertTrue(resp.ok)
         self.assertRequestBodyIs(json={'hello': 'world'})
@@ -57,7 +59,7 @@ class SessionTests(utils.TestCase):
         self.stub_url('HEAD')
         resp = session.head(self.TEST_URL)
 
-        self.assertEqual('HEAD', self.requests.last_request.method)
+        self.assertEqual('HEAD', self.requests_mock.last_request.method)
         self.assertTrue(resp.ok)
         self.assertRequestBodyIs('')
 
@@ -66,7 +68,7 @@ class SessionTests(utils.TestCase):
         self.stub_url('PUT', text='response')
         resp = session.put(self.TEST_URL, json={'hello': 'world'})
 
-        self.assertEqual('PUT', self.requests.last_request.method)
+        self.assertEqual('PUT', self.requests_mock.last_request.method)
         self.assertEqual(resp.text, 'response')
         self.assertTrue(resp.ok)
         self.assertRequestBodyIs(json={'hello': 'world'})
@@ -76,7 +78,7 @@ class SessionTests(utils.TestCase):
         self.stub_url('DELETE', text='response')
         resp = session.delete(self.TEST_URL)
 
-        self.assertEqual('DELETE', self.requests.last_request.method)
+        self.assertEqual('DELETE', self.requests_mock.last_request.method)
         self.assertTrue(resp.ok)
         self.assertEqual(resp.text, 'response')
 
@@ -85,7 +87,7 @@ class SessionTests(utils.TestCase):
         self.stub_url('PATCH', text='response')
         resp = session.patch(self.TEST_URL, json={'hello': 'world'})
 
-        self.assertEqual('PATCH', self.requests.last_request.method)
+        self.assertEqual('PATCH', self.requests_mock.last_request.method)
         self.assertTrue(resp.ok)
         self.assertEqual(resp.text, 'response')
         self.assertRequestBodyIs(json={'hello': 'world'})
@@ -202,7 +204,7 @@ class SessionTests(utils.TestCase):
             m.assert_called_with(2.0)
 
         # we count retries so there will be one initial request + 3 retries
-        self.assertThat(self.requests.request_history,
+        self.assertThat(self.requests_mock.request_history,
                         matchers.HasLength(retries + 1))
 
     def test_uses_tcp_keepalive_by_default(self):
@@ -218,6 +220,52 @@ class SessionTests(utils.TestCase):
         client_session.Session(session=mock_session)
         self.assertFalse(mock_session.mount.called)
 
+    def test_ssl_error_message(self):
+        error = uuid.uuid4().hex
+
+        def _ssl_error(request, context):
+            raise requests.exceptions.SSLError(error)
+
+        self.stub_url('GET', text=_ssl_error)
+        session = client_session.Session()
+
+        # The exception should contain the URL and details about the SSL error
+        msg = _('SSL exception connecting to %(url)s: %(error)s') % {
+            'url': self.TEST_URL, 'error': error}
+        self.assertRaisesRegexp(exceptions.SSLError,
+                                msg,
+                                session.get,
+                                self.TEST_URL)
+
+    def test_mask_password_in_http_log_response(self):
+        session = client_session.Session()
+
+        def fake_debug(msg):
+            self.assertNotIn('verybadpass', msg)
+
+        logger = mock.Mock(isEnabledFor=mock.Mock(return_value=True))
+        logger.debug = mock.Mock(side_effect=fake_debug)
+        body = {
+            "connection_info": {
+                "driver_volume_type": "iscsi",
+                "data": {
+                    "auth_password": "verybadpass",
+                    "target_discovered": False,
+                    "encrypted": False,
+                    "qos_specs": None,
+                    "target_iqn": ("iqn.2010-10.org.openstack:volume-"
+                                   "744d2085-8e78-40a5-8659-ef3cffb2480e"),
+                    "target_portal": "172.99.69.228:3260",
+                    "volume_id": "744d2085-8e78-40a5-8659-ef3cffb2480e",
+                    "target_lun": 1,
+                    "access_mode": "rw",
+                    "auth_username": "verybadusername",
+                    "auth_method": "CHAP"}}}
+        body_json = jsonutils.dumps(body)
+        response = mock.Mock(text=body_json, status_code=200, headers={})
+        session._http_log_response(response=response, logger=logger)
+        self.assertEqual(1, logger.debug.call_count)
+
 
 class RedirectTests(utils.TestCase):
 
@@ -234,14 +282,14 @@ class RedirectTests(utils.TestCase):
         redirect_kwargs.setdefault('text', self.DEFAULT_REDIRECT_BODY)
 
         for s, d in zip(self.REDIRECT_CHAIN, self.REDIRECT_CHAIN[1:]):
-            self.requests.register_uri(method, s, status_code=status_code,
-                                       headers={'Location': d},
-                                       **redirect_kwargs)
+            self.requests_mock.register_uri(method, s, status_code=status_code,
+                                            headers={'Location': d},
+                                            **redirect_kwargs)
 
         final_kwargs.setdefault('status_code', 200)
         final_kwargs.setdefault('text', self.DEFAULT_RESP_BODY)
-        self.requests.register_uri(method, self.REDIRECT_CHAIN[-1],
-                                   **final_kwargs)
+        self.requests_mock.register_uri(method, self.REDIRECT_CHAIN[-1],
+                                        **final_kwargs)
 
     def assertResponse(self, resp):
         self.assertEqual(resp.status_code, 200)
@@ -405,7 +453,7 @@ class SessionAuthTests(utils.TestCase):
         base_url = AuthPlugin.SERVICE_URLS[service_type][interface]
         uri = "%s/%s" % (base_url.rstrip('/'), path.lstrip('/'))
 
-        self.requests.register_uri(method, uri, **kwargs)
+        self.requests_mock.register_uri(method, uri, **kwargs)
 
     def test_auth_plugin_default_with_plugin(self):
         self.stub_url('GET', base_url=self.TEST_URL, json=self.TEST_JSON)
@@ -446,7 +494,7 @@ class SessionAuthTests(utils.TestCase):
                         endpoint_filter={'service_type': service_type,
                                          'interface': interface})
 
-        self.assertEqual(self.requests.last_request.url,
+        self.assertEqual(self.requests_mock.last_request.url,
                          AuthPlugin.SERVICE_URLS['compute']['public'] + path)
         self.assertEqual(resp.text, body)
         self.assertEqual(resp.status_code, status)
@@ -468,7 +516,7 @@ class SessionAuthTests(utils.TestCase):
     def test_raises_exc_only_when_asked(self):
         # A request that returns a HTTP error should by default raise an
         # exception by default, if you specify raise_exc=False then it will not
-        self.requests.get(self.TEST_URL, status_code=401)
+        self.requests_mock.get(self.TEST_URL, status_code=401)
 
         sess = client_session.Session()
         self.assertRaises(exceptions.Unauthorized, sess.get, self.TEST_URL)
@@ -480,8 +528,8 @@ class SessionAuthTests(utils.TestCase):
         passed = CalledAuthPlugin()
         sess = client_session.Session()
 
-        self.requests.get(CalledAuthPlugin.ENDPOINT + 'path',
-                          status_code=200)
+        self.requests_mock.get(CalledAuthPlugin.ENDPOINT + 'path',
+                               status_code=200)
         endpoint_filter = {'service_type': 'identity'}
 
         # no plugin with authenticated won't work
@@ -504,8 +552,8 @@ class SessionAuthTests(utils.TestCase):
 
         sess = client_session.Session(fixed)
 
-        self.requests.get(CalledAuthPlugin.ENDPOINT + 'path',
-                          status_code=200)
+        self.requests_mock.get(CalledAuthPlugin.ENDPOINT + 'path',
+                               status_code=200)
 
         resp = sess.get('path', auth=passed,
                         endpoint_filter={'service_type': 'identity'})
@@ -537,9 +585,9 @@ class SessionAuthTests(utils.TestCase):
         auth = CalledAuthPlugin(invalidate=True)
         sess = client_session.Session(auth=auth)
 
-        self.requests.get(self.TEST_URL,
-                          [{'text': 'Failed', 'status_code': 401},
-                           {'text': 'Hello', 'status_code': 200}])
+        self.requests_mock.get(self.TEST_URL,
+                               [{'text': 'Failed', 'status_code': 401},
+                                {'text': 'Hello', 'status_code': 200}])
 
         # allow_reauth=True is the default
         resp = sess.get(self.TEST_URL, authenticated=True)
@@ -552,9 +600,9 @@ class SessionAuthTests(utils.TestCase):
         auth = CalledAuthPlugin(invalidate=True)
         sess = client_session.Session(auth=auth)
 
-        self.requests.get(self.TEST_URL,
-                          [{'text': 'Failed', 'status_code': 401},
-                           {'text': 'Hello', 'status_code': 200}])
+        self.requests_mock.get(self.TEST_URL,
+                               [{'text': 'Failed', 'status_code': 401},
+                                {'text': 'Hello', 'status_code': 200}])
 
         self.assertRaises(exceptions.Unauthorized, sess.get, self.TEST_URL,
                           authenticated=True, allow_reauth=False)
@@ -569,14 +617,14 @@ class SessionAuthTests(utils.TestCase):
         override_url = override_base + path
         resp_text = uuid.uuid4().hex
 
-        self.requests.get(override_url, text=resp_text)
+        self.requests_mock.get(override_url, text=resp_text)
 
         resp = sess.get(path,
                         endpoint_override=override_base,
                         endpoint_filter={'service_type': 'identity'})
 
         self.assertEqual(resp_text, resp.text)
-        self.assertEqual(override_url, self.requests.last_request.url)
+        self.assertEqual(override_url, self.requests_mock.last_request.url)
 
         self.assertTrue(auth.get_token_called)
         self.assertFalse(auth.get_endpoint_called)
@@ -589,14 +637,14 @@ class SessionAuthTests(utils.TestCase):
         url = self.TEST_URL + path
 
         resp_text = uuid.uuid4().hex
-        self.requests.get(url, text=resp_text)
+        self.requests_mock.get(url, text=resp_text)
 
         resp = sess.get(url,
                         endpoint_override='http://someother.url',
                         endpoint_filter={'service_type': 'identity'})
 
         self.assertEqual(resp_text, resp.text)
-        self.assertEqual(url, self.requests.last_request.url)
+        self.assertEqual(url, self.requests_mock.last_request.url)
 
         self.assertTrue(auth.get_token_called)
         self.assertFalse(auth.get_endpoint_called)
@@ -608,6 +656,34 @@ class SessionAuthTests(utils.TestCase):
         self.assertEqual(auth.TEST_USER_ID, sess.get_user_id())
         self.assertEqual(auth.TEST_PROJECT_ID, sess.get_project_id())
 
+    def test_logger_object_passed(self):
+        logger = logging.getLogger(uuid.uuid4().hex)
+        logger.setLevel(logging.DEBUG)
+        logger.propagate = False
+
+        io = six.StringIO()
+        handler = logging.StreamHandler(io)
+        logger.addHandler(handler)
+
+        auth = AuthPlugin()
+        sess = client_session.Session(auth=auth)
+        response = uuid.uuid4().hex
+
+        self.stub_url('GET',
+                      text=response,
+                      headers={'Content-Type': 'text/html'})
+
+        resp = sess.get(self.TEST_URL, logger=logger)
+
+        self.assertEqual(response, resp.text)
+        output = io.getvalue()
+
+        self.assertIn(self.TEST_URL, output)
+        self.assertIn(response, output)
+
+        self.assertNotIn(self.TEST_URL, self.logger.output)
+        self.assertNotIn(response, self.logger.output)
+
 
 class AdapterTest(utils.TestCase):
 
@@ -718,12 +794,12 @@ class AdapterTest(utils.TestCase):
         adpt = adapter.Adapter(sess, endpoint_override=endpoint_override)
 
         response = uuid.uuid4().hex
-        self.requests.get(endpoint_url, text=response)
+        self.requests_mock.get(endpoint_url, text=response)
 
         resp = adpt.get(path)
 
         self.assertEqual(response, resp.text)
-        self.assertEqual(endpoint_url, self.requests.last_request.url)
+        self.assertEqual(endpoint_url, self.requests_mock.last_request.url)
 
         self.assertEqual(endpoint_override, adpt.get_endpoint())
 
@@ -760,7 +836,7 @@ class AdapterTest(utils.TestCase):
             self.assertEqual(retries, m.call_count)
 
         # we count retries so there will be one initial request + 2 retries
-        self.assertThat(self.requests.request_history,
+        self.assertThat(self.requests_mock.request_history,
                         matchers.HasLength(retries + 1))
 
     def test_user_and_project_id(self):
@@ -771,6 +847,35 @@ class AdapterTest(utils.TestCase):
         self.assertEqual(auth.TEST_USER_ID, adpt.get_user_id())
         self.assertEqual(auth.TEST_PROJECT_ID, adpt.get_project_id())
 
+    def test_logger_object_passed(self):
+        logger = logging.getLogger(uuid.uuid4().hex)
+        logger.setLevel(logging.DEBUG)
+        logger.propagate = False
+
+        io = six.StringIO()
+        handler = logging.StreamHandler(io)
+        logger.addHandler(handler)
+
+        auth = AuthPlugin()
+        sess = client_session.Session(auth=auth)
+        adpt = adapter.Adapter(sess, auth=auth, logger=logger)
+
+        response = uuid.uuid4().hex
+
+        self.stub_url('GET', text=response,
+                      headers={'Content-Type': 'text/html'})
+
+        resp = adpt.get(self.TEST_URL, logger=logger)
+
+        self.assertEqual(response, resp.text)
+        output = io.getvalue()
+
+        self.assertIn(self.TEST_URL, output)
+        self.assertIn(response, output)
+
+        self.assertNotIn(self.TEST_URL, self.logger.output)
+        self.assertNotIn(response, self.logger.output)
+
 
 class ConfLoadingTests(utils.TestCase):
 
