@@ -15,6 +15,7 @@ from keystoneclient.v3.contrib.federation import identity_providers
 from keystoneclient.v3.contrib.federation import mappings
 from keystoneclient.v3.contrib.federation import projects
 from keystoneclient.v3.contrib.federation import protocols
+from keystoneclient.v3.contrib.federation import service_providers
 
 
 class FederationManager(object):
@@ -25,3 +26,4 @@ class FederationManager(object):
         self.protocols = protocols.ProtocolManager(api)
         self.projects = projects.ProjectManager(api)
         self.domains = domains.DomainManager(api)
+        self.service_providers = service_providers.ServiceProviderManager(api)
